@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: forms.py 11753 2019-10-20 17:58:13Z Lavender $
+# $Id: forms.py 11758 2019-10-22 08:06:50Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $ (last)
-# $Date: 2019-10-21 02:58:13 +0900 (週一, 21 十月 2019) $
-# $Revision: 11753 $
+# $Date: 2019-10-22 17:06:50 +0900 (週二, 22 十月 2019) $
+# $Revision: 11758 $
 
 import os
 import hashlib
@@ -26,6 +26,7 @@ from django.contrib import messages
 from django.core.files.storage import default_storage
 from django.core.files.base import ContentFile
 from django.conf import settings
+from django.db.models import Q
 
 from oscar import VERSION
 from oscar.core.loading import get_model, get_class, get_classes
@@ -120,7 +121,106 @@ def apply_search(self, queryset):
     
 catalogueViews.ProductListView.form_class = ProductSearchForm
 catalogueViews.ProductListView.apply_search = apply_search
+catalogueViews.ProductListView.template_name = \
+    'shop/dashboard/catalogue/product_list.html'
+    
+# customer search form
+_UserSearchForm = get_class('dashboard.users.forms', 'UserSearchForm')
+usersViews = get_class('dashboard.users', 'views')
 
+class UserSearchForm(_UserSearchForm):
+    notActive = forms.BooleanField(
+        label=_('Not active'), required=False, initial=False)
+    notStaff = forms.BooleanField(
+        label=_('Not staff status'), required=False, initial=False)
+    dateFrom = forms.DateField(
+        required=False, label=_("Date from"), widget=DatePickerInput)
+    dateTo = forms.DateField(
+        required=False, label=_("Date to"), widget=DatePickerInput)
+    
+_apply_search2 = usersViews.IndexView.apply_search
+def apply_search2(self, queryset):
+    self.form = self.form_class(self.request.GET)
+    if not self.form.is_valid():
+        return queryset
+    data = self.form.cleaned_data
+    queryset = _apply_search2(self, queryset)
+    
+    if data.get('notActive'):
+        queryset = queryset.filter(is_active=False)
+    
+    if data.get('notStaff'):
+        queryset = queryset.filter(is_staff=False)
+    
+    
+    if data['dateFrom'] and data['dateTo']:
+        dateTo = datetime_combine(data['dateTo'], datetime.time.max)
+        dateFrom = datetime_combine(data['dateFrom'], datetime.time.min)
+        queryset = queryset.filter(
+            date_joined__gte=dateFrom, ddate_joined__lt=dateTo)
+    elif data['dateFrom']:
+        dateFrom = datetime_combine(data['dateFrom'], datetime.time.min)
+        queryset = queryset.filter(date_joined__gte=dateFrom)
+    elif data['dateTo']:
+        dateTo = datetime_combine(data['dateTo'], datetime.time.max)
+        queryset = queryset.filter(date_joined__lt=dateTo)
+    
+    return queryset
+
+usersViews.IndexView.form_class = UserSearchForm
+usersViews.IndexView.apply_search = apply_search2
+usersViews.IndexView.template_name = 'shop/dashboard/users/index.html'
+
+# https://code.nuwainfo.com/trac/Mercurius/ticket/3278#ticket
+_ProductRecommendationForm = get_class(
+    'dashboard.catalogue.forms', 'ProductRecommendationForm')
+ProductSelect = get_class('dashboard.catalogue.widgets', 'ProductSelect')
+
+_lookup_filter = catalogueViews.ProductLookupView.lookup_filter
+def lookup_filter(self, qs, term):
+    return qs.filter(Q(title__icontains=term)
+                     | Q(parent__title__icontains=term)
+                     | Q(upc__icontains=term))
+def format_object(self, obj):
+    return {
+        'id': obj.pk,
+        'text': u"%s (%s)" % (obj, obj.upc),
+    }
+                     
+catalogueViews.ProductLookupView.lookup_filter = lookup_filter
+catalogueViews.ProductLookupView.format_object = format_object
+
+# https://code.nuwainfo.com/trac/Mercurius/ticket/3272#ticket
+def process_all_forms(self, form):
+    if self.creating and form.is_valid():
+        self.object = form.save(commit=False)
+    
+    postCopy = self.request.POST.copy()
+    for k, v in postCopy.items():
+        if 'options-' in k and '-option' in k and v:
+            postCopy[k] = u"%s" % hashlib.md5(
+                str("%s%s" % (v, datetime.datetime.now())).encode()).hexdigest()
+    
+    attribute_option_formset = self.attribute_option_formset(
+        self.request.POST, self.request.FILES, instance=self.object)
+    attribute_option_formset2 = self.attribute_option_formset(
+        postCopy, self.request.FILES, instance=self.object)
+
+    is_valid = form.is_valid() and attribute_option_formset2.is_valid()
+
+    if is_valid:
+        attribute_option_formset2.save()
+        return self.forms_valid(form, attribute_option_formset)
+    else:
+        return self.forms_invalid(form, attribute_option_formset)
+        
+catalogueViews.AttributeOptionGroupCreateUpdateView.process_all_forms = \
+    process_all_forms
+catalogueViews.AttributeOptionGroupCreateUpdateView.form_valid = \
+    process_all_forms
+catalogueViews.AttributeOptionGroupCreateUpdateView.form_invalid = \
+    process_all_forms
+    
 # content block ckeditor
 if not VERSION >= (2, 0, 0):
     # RawHTMLForm
