@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: admin.py 11449 2019-04-13 12:42:08Z Lavender $
+# $Id: admin.py 11699 2019-09-05 08:41:06Z Lavender $
 #
 # Copyright (c) 2017 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2019-04-13 20:42:08 +0800 (週六, 13 四月 2019) $
-# $Revision: 11449 $
+# $Date: 2019-09-05 17:41:06 +0900 (週四, 05 九月 2019) $
+# $Revision: 11699 $
 
 import os
 import datetime
@@ -97,44 +97,71 @@ def createVersion(model, node, conflict=False):
         else:
             backupName = "%s_%d" % (fileName, maxVersion + 1)
 
-        if not preVersion.checksum == checksum:
-            preVersionUpdateTime = datetime.datetime.fromtimestamp(
-                os.stat(preVersion.absoluteBackupPath).st_mtime)
-            currentFileUpdateTime = datetime.datetime.fromtimestamp(
-                os.stat(fullPath).st_mtime)
-            
-            if preVersionUpdateTime < currentFileUpdateTime: 
-                # 當前檔案比版控最新版本的新時，建立新版本並判斷是否衝突
-                if not maxVersion == 0 or not conflict:
-                    version = Version.objects.create(
-                        dirName=dirName,
-                        checksum=checksum,
-                        number=maxVersion + 1,
-                        fileName=backupName,
-                        conflict=conflict,
-                        preVersion=preVersion,
-                    )
-                    preVersion.nextVersion = version
-                    preVersion.save()
-                else:
-                    preVersion.checksum = checksum
-                    preVersion.save()
-                    if subFileName:
-                        backupName = "%s_%d.%s" % (
-                            fileName, maxVersion, subFileName)
+        if os.path.exists(preVersion.absoluteBackupPath):
+            if not preVersion.checksum == checksum:
+                preVersionUpdateTime = datetime.datetime.fromtimestamp(
+                    os.stat(preVersion.absoluteBackupPath).st_mtime)
+                currentFileUpdateTime = datetime.datetime.fromtimestamp(
+                    os.stat(fullPath).st_mtime)
+                
+                if preVersionUpdateTime < currentFileUpdateTime: 
+                    # 當前檔案比版控最新版本的新時，建立新版本並判斷是否衝突
+                    if not maxVersion == 0 or not conflict:
+                        version = Version.objects.create(
+                            dirName=dirName,
+                            checksum=checksum,
+                            number=maxVersion + 1,
+                            fileName=backupName,
+                            conflict=conflict,
+                            preVersion=preVersion,
+                        )
+                        preVersion.nextVersion = version
+                        preVersion.save()
                     else:
-                        backupName = "%s_%d" % (fileName, maxVersion)
+                        preVersion.checksum = checksum
+                        preVersion.save()
+                        if subFileName:
+                            backupName = "%s_%d.%s" % (
+                                fileName, maxVersion, subFileName)
+                        else:
+                            backupName = "%s_%d" % (fileName, maxVersion)
 
+                        version = preVersion
+
+                    createFile = True
+                else:
+                    # 當前檔案比版控最新版本的舊時，不建新 Version，並且換回最新版控
                     version = preVersion
-
-                createFile = True
+                    shutil.copyfile(preVersion.absoluteBackupPath, fullPath)
             else:
-                # 當前檔案比版控最新版本的舊時，不建新 Version，並且換回最新版控
                 version = preVersion
-                shutil.copyfile(preVersion.absoluteBackupPath, fullPath)
         else:
-            version = preVersion
+            # 備分資料夾意外被刪除，自動刪除對不上的版本
+            logger.warning(
+                "Template Editor: %s's version backup files had be deleted. "
+                "create new versions for %s" % (node.name, node.name)
+            )
+            
+            # delete all version
+            Version.objects.filter(dirName=dirName).delete()
+            
+            # 重新建 version
+            maxVersion = 0
+            
+            if subFileName:
+                backupName = "%s_%d.%s" % (fileName, maxVersion, subFileName)
+            else:
+                backupName = "%s_%d" % (fileName, maxVersion)
+                
+            version = Version.objects.create(
+                dirName=dirName,
+                checksum=checksum,
+                number=maxVersion,
+                fileName=backupName,
+            )
+            createFile = True 
     else:
+        # 沒有 version，建 version
         maxVersion = 0
         if subFileName:
             backupName = "%s_%d.%s" % (fileName, maxVersion, subFileName)
@@ -163,6 +190,10 @@ def createVersion(model, node, conflict=False):
 
 @transaction.atomic
 def checkVersion(model):
+    # 確保 BACKUP_FILE_DIR 存在
+    if not os.path.isdir(BACKUP_FILE_DIR):
+        os.mkdir(BACKUP_FILE_DIR)
+        
     conflictNodeList = []
     for node in model.objects.filter(type=FILE):
         version = createVersion(model, node, conflict=True)
@@ -198,15 +229,19 @@ def loginCheck(sender, user, request, **kwargs):
                 }
             return mark_safe(msg)
 
-        conflictNodeList = checkVersion(Static)
-        for ele in conflictNodeList:
-            msg = createMessage(ele, Static)
-            messages.warning(request, msg)
+        try:
+            conflictNodeList = checkVersion(Static)
+            for ele in conflictNodeList:
+                msg = createMessage(ele, Static)
+                messages.warning(request, msg)
 
-        conflictNodeList = checkVersion(Template)
-        for ele in conflictNodeList:
-            msg = createMessage(ele, Template)
-            messages.warning(request, msg)
+            conflictNodeList = checkVersion(Template)
+            for ele in conflictNodeList:
+                msg = createMessage(ele, Template)
+                messages.warning(request, msg)
+        except Exception as e:
+            logger.error(str(e) + ", please try use build_tree and "
+                "clean_version command to fix it.")
 
 user_logged_in.connect(loginCheck)
 
